use crate::{key::KeyType, schema, UIDCError};
use microrm::prelude::*;

pub fn create(
    realm: &microrm::Stored<schema::Realm>,
    name: &String,
    key_type: KeyType,
) -> Result<(), UIDCError> {
    let rng = ring::rand::SystemRandom::new();
    let client_secret: [u8; 32] = ring::rand::generate(&rng).unwrap().expose();
    let refresh_token_secret: [u8; 32] = ring::rand::generate(&rng).unwrap().expose();

    realm.clients.insert(schema::Client {
        realm: realm.id(),
        shortname: name.into(),
        secret: base64::encode(&client_secret),
        access_key_type: key_type.into(),
        refresh_key_type: key_type.into(),
        refresh_token_secret: refresh_token_secret.into(),
        redirects: Default::default(),
        scopes: Default::default(),
    })?;
    Ok(())
}

pub fn rotate_secret(realm: &microrm::Stored<schema::Realm>, name: &str) -> Result<(), UIDCError> {
    let rng = ring::rand::SystemRandom::new();
    let client_secret: [u8; 32] = ring::rand::generate(&rng).unwrap().expose();

    let mut client = realm
        .clients
        .with(schema::Client::Shortname, name)
        .first()
        .get()?
        .ok_or(UIDCError::Abort("no such client"))?;
    client.secret = base64::encode(&client_secret);

    client.sync()?;

    Ok(())
}
